export declare type State = Record<string | number | symbol, any>;
export declare type StateListener<T> = (state: T) => void;
export declare type StateSelector<T extends State, U> = (state: T) => U;
export declare type PartialState<T extends State> = Partial<T> | ((state: T) => Partial<T>);
export declare type EqualityChecker<T> = (state: T, newState: any) => boolean;
export interface SubscribeOptions<T extends State, U> {
    selector?: StateSelector<T, U>;
    equalityFn?: EqualityChecker<U>;
    currentSlice?: U;
    subscribeError?: Error;
}
export declare type StateCreator<T extends State> = (set: SetState<T>, get: GetState<T>, api: StoreApi<T>) => T;
export declare type SetState<T extends State> = (partial: PartialState<T>) => void;
export declare type GetState<T extends State> = () => T;
export declare type Subscribe<T extends State> = <U>(listener: StateListener<U | void>, options?: SubscribeOptions<T, U>) => () => void;
export declare type Destroy = () => void;
export declare type UseStore<T extends State> = <U>(selector?: StateSelector<T, U>, equalityFn?: EqualityChecker<U>) => U;
export interface StoreApi<T extends State> {
    setState: SetState<T>;
    getState: GetState<T>;
    subscribe: Subscribe<T>;
    destroy: Destroy;
}
export default function create<TState extends State>(createState: keyof TState extends never ? StateCreator<State> : StateCreator<TState>): [UseStore<TState>, StoreApi<TState>];
export { create };
